@@ -1,7 +1,7 @@
 /*
  * Copyright © 2009 CNRS
  * Copyright © 2009-2013 inria.  All rights reserved.
- * Copyright © 2009-2011 Université Bordeaux 1
+ * Copyright © 2009-2011 Université Bordeaux
  * Copyright © 2011 Cisco Systems, Inc.  All rights reserved.
  * See COPYING in top-level directory.
  */
@@ -12,15 +12,14 @@
  * Applications that use both hwloc and glibc scheduling routines such as
  * sched_getaffinity() or pthread_attr_setaffinity_np() may want to include
  * this file so as to ease conversion between their respective types.
- *
- * \note Topology \p topology must match the current machine.
  */
 
 #ifndef HWLOC_GLIBC_SCHED_H
 #define HWLOC_GLIBC_SCHED_H
 
-#include <hwloc.h>
-#include <hwloc/helper.h>
+#include "hwloc.h"
+#include "hwloc/helper.h"
+
 #include <assert.h>
 
 #if !defined _GNU_SOURCE || !defined _SCHED_H || (!defined CPU_SETSIZE && !defined sched_priority)
@@ -36,7 +35,13 @@ extern "C" {
 #ifdef HWLOC_HAVE_CPU_SET
 
 
-/** \defgroup hwlocality_glibc_sched Helpers for manipulating glibc sched affinity
+/** \defgroup hwlocality_glibc_sched Interoperability with glibc sched affinity
+ *
+ * This interface offers ways to convert between hwloc cpusets and glibc cpusets
+ * such as those manipulated by sched_getaffinity() or pthread_attr_setaffinity_np().
+ *
+ * \note Topology \p topology must match the current machine.
+ *
  * @{
  */
 
